from typing import Dict

class DataSources:
    """Registry of supported data sources based on STAC endpoint collection names."""

    LANDSAT = "landsat-c2l2-sr"
    SENTINEL2 = "sentinel-2-l2a"

    LANDSATC2L2 = "landsat-c2-l2"
    JRCGSW = "jrc-gsw"

    @classmethod
    def list_sources(cls) -> Dict[str, str]:
        """List available data sources with descriptions."""
        return {
            cls.LANDSAT: "Landsat Collection 2 Level 2 Surface Reflectance",
            cls.SENTINEL2: "Sentinel-2 Level 2A",
            cls.LANDSATC2L2: "Landsat Collection 2 Level 2 IN Microsoft"
        }

SENTINEL2_BANDS: Dict[str, str] = {
    "B01": "coastal",
    "B02": "blue",
    "B03": "green",
    "B04": "red",
    "B05": "rededge1",
    "B06": "rededge2",
    "B07": "rededge3",
    "B08": "nir",
    "B8A": "nir08",
    "B09": "nir09",
    "B11": "swir16",
    "B12": "swir22",
    "SCL": "scl",
}

SENTINEC2L2_BANDS: Dict[str, str] = {
    "OLI_B4": "red",
    "OLI_B2": "blue",
    "OLI_B3": "green",
    "OLI_B5": "nir08",
    "OLI_B6": "swir16",
    "OLI_B7": "swir22",
    "OLI_B1": "coastal"
}

LANDSAT9_BANDS: Dict[str, str] = {
    "B1": "coastal",
    "B2": "blue",
    "B3": "green",
    "B4": "red",
    "B5": "nir08",
    "B6": "swir16",
    "B7": "swir22",
    "qa_aerosol": "qa_aerosol",
    "qa_pixel": "qa_pixel",
    "qa_radsat": "qa_radsat",
}

LANDSATC2L2_BANDS_SORT = ["B1", "B2", "B3", "B4", "B5", "B6", "B7", "B10", "ATRAN", "CDIST", "DRAD", "EMIS", "EMSD", "QA", "QA_AEROSOL", "QA_PIXEL", "QA_RADSAT"]

LANDSATC2L2_BANDS: Dict[str, str] = {
    "coastal": "B1",
    "blue": "B2",
    "green": "B3",
    "red": "B4",
    "nir08": "B5",
    "swir16": "B6",
    "swir22": "B7",
    "lwir11": "B10",
    "atran": "ATRAN",
    "cdist": "CDIST",
    "drad": "DRAD",
    "emis": "EMIS",
    "emsd": "EMSD",
    "qa": "QA",
    "qa_aerosol": "QA_AEROSOL",
    "qa_pixel": "QA_PIXEL",
    "qa_radsat": "QA_RADSAT",
    "trad": "TRAD",
    "urad": "URAD"
    }

STAC_COLLECTION_BAND_MAPS = {
    DataSources.SENTINEL2: SENTINEL2_BANDS,
    DataSources.LANDSAT: LANDSAT9_BANDS,
    "LANDSAT": LANDSAT9_BANDS,
    "SENTINEL2": SENTINEL2_BANDS,
    DataSources.LANDSATC2L2: LANDSATC2L2_BANDS,
}

EPSG_3857 = "EPSG:3857"
EPSG_3857_NUM = "3857"
EPSG_4326 = "EPSG:4326"
EPSG_4326_NUM = "4326"